import java.lang.Math;
import java.awt.Color;
import jrobots.utils.*;

import jrobots.simulation.simulationObjects.JRobot2010;

/**
 * Sozusagen ein Starter-Toolkit. 
 * <p>Er enth&auml;lt schon einen rundum-Scanner und kann auch schon fahren. 
 * <p>...Was? Er schie&szlig;t nicht?
 * <p>Vielleicht sollte man das mal &auml;ndern. Und nicht vergessen: Gebt
 * ihm auch einen sch&ouml;nen Namen.
 */
public class OnsetBot2010 extends JRobot2010 {

    private Angle scanAperture; //momentary scanAperture, needed in next frame 
    private Angle scanDirection; //momentary scanDirection, needed in next frame 
    private Scan latestSuccessfulScan; //last scanned target
    private int scanState;
    private int scan_ROUNDABOUT_COUNT;
    
    /**
     * Sensoren auswerten
     */
    private void collect() {
        // Scanner nur auswerten, wenn gerade gescannt wurde
        if (isScanFromNow()) {
            if (this.getLastScan().isTargetLocated()) {
                latestSuccessfulScan = this.getLastScan();
            }
        }
    }
    
    /**
     * Wohin soll der Scanner ausgerichtet werden?
     */
    private void scan() {
        // Scanner neu ausrichten, wenn gerade gescannt wurde
        if (isScanFromNow()) {
            // 360 Grad scannen
            
            // naechster Scannerzustand
            scanState++; 
            // wegen modulo gibt's keinen Ueberlauf
            scanState = scanState % scan_ROUNDABOUT_COUNT; 
            // maximaler Oeffnungswinkel
            scanAperture = getMaxScanAperture();
            // Scanrichtung: #Scan * (360 Grad / gesamtanzahl_scans)
            scanDirection = (new Angle(360, "Degrees"))
                    .mult((double)scanState / scan_ROUNDABOUT_COUNT);
            // Oeffnungswinkel und Scanrichtung an Scanner durchgeben
            setScanAperture(scanAperture);
            setScanDirection(scanDirection);
            
        } else {
            // Wenn noch nicht gescannt wurde, zur Sicherheit nochmal Befehl setzen
            setScanAperture(scanAperture);
            setScanDirection(scanDirection);
        }
    }
    
    /**
     * Wohin soll geschossen werden?
     */
    private void shoot() {
    }
    
    /**
     * Wohin f&auml;hrt der JRobot?
     */
    private void drive() {
        // in Richtung des letzten gescannten Gegners fahren       
        setAutopilot(latestSuccessfulScan.scanDirection, 1.0);
    }
    
    
    @Override
    protected void actions() {
        calibrate(); //should be first
        
        collect(); //must be before scan
        
        scan(); //must be before drive
        
        shoot();
        
        drive();
    }
    
    @Override
    protected void init() {
        setNameColor(new Color((float)0.0,(float)0.0,(float)0.0));
        setAutopilot(getOrientation(), 1.0);
        scanAperture = getMaxScanAperture();
        scanDirection = getOrientation();
        setScanAperture(scanAperture);
        setScanDirection(scanDirection);
        latestSuccessfulScan = new Scan(
                RIGHT, 
                new Angle(360, "d"),
                Double.MAX_VALUE,
                this.getPosition(),
                Double.NEGATIVE_INFINITY);
    }
    
    /**
     * Hier berechnet der Bot aus den "Konstanten" der Umwelt abgeleitete Werte, 
     * die er in seinen Routinen <code>scan()</code>, <code>shoot()</code> und
     * <code>drive()</code> verwendet.  
     */
    private void calibrate() {
        scan_ROUNDABOUT_COUNT = Math.max(3, 
             (int)Math.ceil(360.0 / getMaxScanAperture().getValueAsDegrees()));
    }

}
